@@ -6,13 +6,13 @@
  * Version 2 by Charles Cazabon <charlesc-memtester@pyropus.ca>
  * Version 3 not publicly released.
  * Version 4 rewrite:
- * Copyright (C) 2004-2020 Charles Cazabon <charlesc-memtester@pyropus.ca>
+ * Copyright (C) 2004-2024 Charles Cazabon <charlesc-memtester@pyropus.ca>
  * Licensed under the terms of the GNU General Public License version 2 (only).
  * See the file COPYING for details.
  *
  */
 
-#define __version__ "4.6.0"
+#define __version__ "4.7.0"
 
 #include <stddef.h>
 #include <stdlib.h>
@@ -50,7 +50,7 @@ struct test tests[] = {
     { "Bit Flip", test_bitflip_comparison },
     { "Walking Ones", test_walkbits1_comparison },
     { "Walking Zeroes", test_walkbits0_comparison },
-#ifdef TEST_NARROW_WRITES    
+#ifdef TEST_NARROW_WRITES
     { "8-bit Writes", test_8bit_wide_random },
     { "16-bit Writes", test_16bit_wide_random },
 #endif
@@ -103,7 +103,7 @@ off_t physaddrbase = 0;
 /* Function definitions */
 void usage(char *me) {
     fprintf(stderr, "\n"
-            "Usage: %s [-p physaddrbase [-d device]] <mem>[B|K|M|G] [loops]\n",
+            "Usage: %s [-p physaddrbase [-d device] [-u]] <mem>[B|K|M|G] [loops]\n",
             me);
     exit(EXIT_FAIL_NONSTARTER);
 }
@@ -127,16 +127,17 @@ int main(int argc, char **argv) {
     int device_specified = 0;
     char *env_testmask = 0;
     ul testmask = 0;
+    int o_flags = O_RDWR | O_SYNC;
 
     printf("memtester version " __version__ " (%d-bit)\n", UL_LEN);
-    printf("Copyright (C) 2001-2020 Charles Cazabon.\n");
+    printf("Copyright (C) 2001-2024 Charles Cazabon.\n");
     printf("Licensed under the GNU General Public License version 2 (only).\n");
     printf("\n");
     check_posix_system();
     pagesize = memtester_pagesize();
     pagesizemask = (ptrdiff_t) ~(pagesize - 1);
     printf("pagesizemask is 0x%tx\n", pagesizemask);
-    
+
     /* If MEMTESTER_TEST_MASK is set, we use its value as a mask of which
        tests we run.
      */
@@ -144,14 +145,14 @@ int main(int argc, char **argv) {
         errno = 0;
         testmask = strtoul(env_testmask, 0, 0);
         if (errno) {
-            fprintf(stderr, "error parsing MEMTESTER_TEST_MASK %s: %s\n", 
+            fprintf(stderr, "error parsing MEMTESTER_TEST_MASK %s: %s\n",
                     env_testmask, strerror(errno));
             usage(argv[0]); /* doesn't return */
         }
         printf("using testmask 0x%lx\n", testmask);
     }
 
-    while ((opt = getopt(argc, argv, "p:d:")) != -1) {
+    while ((opt = getopt(argc, argv, "p:d:u")) != -1) {
         switch (opt) {
             case 'p':
                 errno = 0;
@@ -180,12 +181,12 @@ int main(int argc, char **argv) {
                 break;
             case 'd':
                 if (stat(optarg,&statbuf)) {
-                    fprintf(stderr, "can not use %s as device: %s\n", optarg, 
+                    fprintf(stderr, "can not use %s as device: %s\n", optarg,
                             strerror(errno));
                     usage(argv[0]); /* doesn't return */
                 } else {
                     if (!S_ISCHR(statbuf.st_mode)) {
-                        fprintf(stderr, "can not mmap non-char device %s\n", 
+                        fprintf(stderr, "can not mmap non-char device %s\n",
                                 optarg);
                         usage(argv[0]); /* doesn't return */
                     } else {
@@ -193,18 +194,21 @@ int main(int argc, char **argv) {
                         device_specified = 1;
                     }
                 }
-                break;              
+                break;
+            case 'u':
+	            o_flags &= ~O_SYNC;
+	            break;
             default: /* '?' */
                 usage(argv[0]); /* doesn't return */
         }
     }
 
     if (device_specified && !use_phys) {
-        fprintf(stderr, 
+        fprintf(stderr,
                 "for mem device, physaddrbase (-p) must be specified\n");
         usage(argv[0]); /* doesn't return */
     }
-    
+
     if (optind >= argc) {
         fprintf(stderr, "need memory argument, in MB\n");
         usage(argv[0]); /* doesn't return */
@@ -272,7 +276,7 @@ int main(int argc, char **argv) {
     buf = NULL;
 
     if (use_phys) {
-        memfd = open(device_name, O_RDWR | O_SYNC);
+        memfd = open(device_name, o_flags);
         if (memfd == -1) {
             fprintf(stderr, "failed to open %s for physical memory: %s\n",
                     device_name, strerror(errno));
@@ -362,9 +366,9 @@ int main(int argc, char **argv) {
 
     /* Do alighnment here as well, as some cases won't trigger above if you
        define out the use of mlock() (cough HP/UX 10 cough). */
-    if ((size_t) buf % pagesize) {
+    if ((size_t) aligned % pagesize) {
         /* printf("aligning to page -- was 0x%tx\n", buf); */
-        aligned = (void volatile *) ((size_t) buf & pagesizemask) + pagesize;
+        aligned = (void volatile *) ((size_t) aligned & pagesizemask) + pagesize;
         /* printf("  now 0x%tx -- lost %d bytes\n", aligned,
          *      (size_t) aligned - (size_t) buf);
          */
